/*
Copyright (C) 2008 Jeremy Chadwick. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/


/*
 * A list of hardware monitoring ASIC types.  Every entry in this enum
 * should have a corresponding entry in boardlist[] (see boards.c).
 * Failure to do so could be detrimental.
 */
enum chips_e {
	WINBOND_W83792D,
	WINBOND_W83793G,
	CUSTOM_X6DVA
};

/* Voltage index declared/used across all motherboards. */
enum voltages_e {
	VOLT_VCOREA, VOLT_VCOREB, VOLT_VTT, VOLT_VSEN1, VOLT_VSEN2,
	VOLT_3VSEN, VOLT_12VSEN, VOLT_5VDD, VOLT_5VSB, VOLT_VBAT,
	VOLT_5VCC, VOLT_VIN0, VOLT_VIN1, VOLT_VIN2, VOLT_VIN3,
	VOLT_IC1_P25V, VOLT_IC1_VCCP1, VOLT_IC1_VCC, VOLT_IC1_P5V,
	VOLT_IC1_P12V, VOLT_IC1_VCCP2,
	VOLT_MAX	/* DO NOT DELETE - must come last */
};


/* Temperature/thermistor index declared/used across all motherboards. */
enum temps_e {
        TEMP_TD1, TEMP_TD2, TEMP_TD3, TEMP_TD4, TEMP_TR1, TEMP_TR2,
	TEMP_IC1_INT, TEMP_IC2_INT, TEMP_CPU1, TEMP_CPU2,
	TEMP_MAX	/* DO NOT DELETE - must come last */
};


/* Fan index declared/used across all motherboards. */
enum fans_e {
        FAN_FAN1, FAN_FAN2, FAN_FAN3, FAN_FAN4, FAN_FAN5, FAN_FAN6,
        FAN_FAN7, FAN_FAN8, FAN_FAN9, FAN_FAN10, FAN_FAN11, FAN_FAN12,
	FAN_IC1_FAN1, FAN_IC1_FAN2, FAN_IC2_FAN1, FAN_IC2_FAN2,
	FAN_MAX		/* DO NOT DELETE - must come last */
};


/*
 * The pinmap struct defines two pieces of information: an index value
 * (which refers to one of the above enums), and an ASCII character string
 * (which is what's printed to the user).
 *
 * The pinmap struct is used heavily in boards.c for declaring multiple
 * "types" of boards.  For example, FAN_FAN1 might be populated/assigned
 * inside of collect.c, but it might actually be wired to "FAN2" according
 * to the BIOS.  This provides a string-to-wire/pin mapping structure.
 *
 * It's also used in the boards struct; see below.  :-)
 */
struct pinmap {
	unsigned int index;	/* One of the above enums */
	const char *label;	/* ASCII string that's output to the user */
};

struct voltages_data {
	unsigned int index;
	double value;
};

struct temps_data {
	unsigned int index;
	unsigned int value;
};

struct fans_data {
	unsigned int index;
	unsigned int value;
};

struct sensors {
	struct voltages_data voltages[VOLT_MAX];
	struct temps_data temps[TEMP_MAX];
	struct fans_data fans[FAN_MAX];
};

struct board {
	const char *model;
	int chip;
	int slave;
	struct pinmap *voltages;
	struct pinmap *temps;
	struct pinmap *fans;
};

