/*
Copyright (C) 2008 Jeremy Chadwick. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include "global.h"


/* Function prototypes */
void list_models(struct board *);
void sensors_output(struct board *, struct sensors *);
const char *chip_lookup(int);

/* External functions (main.c) */
extern void VERBOSE(const char *, ...);


const char *chip_lookup(int id)
{
	switch (id) {
		case CUSTOM_X6DVA:	return "X6DVA/X6DVL/X6DAL";
		case WINBOND_W83792D:	return "Winbond W83792D";
		case WINBOND_W83793G:	return "Winbond W83793G";
	}
	return "Unknown";
}


/*
 * list_models(struct board *b)
 *
 * b = Pointers to board struct; see boards.c for a definition
 *
 * Outputs a list of everything in the boardlist[] struct.  This is
 * used when calling the main program with the "-l" argument.
 */
void list_models(struct board *b)
{
	unsigned int i = 0;

	VERBOSE("entering list_models()\n");

	printf("smbios.planar.product     Monitoring IC         Slave Addr\n");
	printf("------------------------  --------------------  ----------\n");
	while (1) {
		/* We're at the end of the structure */
		if (b[i].model == NULL) {
			break;
		}

		/* For boards with multiple SMBus slave addresses or custom
		 * methods, output something a little more sane here
		 *
		 * Otherwise, output the SMBus Slave address as expected.
		 */
		if (b[i].slave == -1) {
			printf("%-24s  %-20s  Custom\n",
				b[i].model,
				chip_lookup(b[i].chip)
			);
		}
		else {
			printf("%-24s  %-20s  0x%02x\n",
				b[i].model,
				chip_lookup(b[i].chip),
				b[i].slave
			);
		}
		i++;
	}
	printf("------------------------  --------------------  ----------\n");

	VERBOSE("exiting list_models()\n");
	exit(0);
}


void sensors_output(struct board *b, struct sensors *s)
{
	unsigned int i;
	unsigned int enum_index;

        VERBOSE("entering sensors_output()\n");

	for (i = 0; b->temps[i].label != NULL; i++) {
		enum_index = b->temps[i].index;

		printf("%-20s %8u C\n",
			b->temps[i].label,
			s->temps[enum_index].value
		);
	}

	for (i = 0; b->fans[i].label != NULL; i++) {
		enum_index = b->fans[i].index;

		printf("%-20s %8u RPM\n",
			b->fans[i].label,
			s->fans[enum_index].value
		);
	}

	for (i = 0; b->voltages[i].label != NULL; i++) {
		enum_index = b->voltages[i].index;

		printf("%-20s %8.3f V\n",
			b->voltages[i].label,
			s->voltages[enum_index].value
		);
        }

        VERBOSE("exiting sensors_output()\n");
}

